///////////////////////////////////////////////////////////////////////////////
//                                                                             
//  Copyright (C) 2008-2012  Artyom Beilis (Tonkikh) <artyomtnk@yahoo.com>     
//                                                                             
//  See accompanying file COPYING.TXT file for licensing details.
//
///////////////////////////////////////////////////////////////////////////////
#ifndef CPPCMS_VIEW_H
#define CPPCMS_VIEW_H
#include <cppcms/form.h>
#include <cppcms/filters.h>
#include <cppcms/base_view.h>
#include <cppcms/views_pool.h>
#include <cppcms/application.h>
#include <cppcms/url_mapper.h>
#include <cppcms/copy_filter.h>
#include <cppcms/cache_interface.h>
#include <cppcms/session_interface.h>
#endif

