//
//  Copyright (c) 2009-2011 Artyom Beilis (Tonkikh)
//
//  Distributed under the Boost Software License, Version 1.0. (See
//  accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)
//
#ifndef BOOSTER_LOCALE_BOUNDARY_H_INCLUDED
#define BOOSTER_LOCALE_BOUNDARY_H_INCLUDED

#include <booster/locale/boundary/types.h>
#include <booster/locale/boundary/facets.h>
#include <booster/locale/boundary/segment.h>
#include <booster/locale/boundary/boundary_point.h>
#include <booster/locale/boundary/index.h>

#endif
// vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
