///////////////////////////////////////////////////////////////////////////////
//                                                                             
//  Copyright (C) 2008-2012  Artyom Beilis (Tonkikh) <artyomtnk@yahoo.com>     
//                                                                             
//  See accompanying file COPYING.TXT file for licensing details.
//
///////////////////////////////////////////////////////////////////////////////
#ifndef CPPCMS_LOCALE_FWD_H
#define CPPCMS_LOCALE_FWD_H

#include <cppcms/config.h>

namespace booster {
	namespace locale {
		class generator;
		class info;
	}
}
namespace cppcms { 
	namespace locale = ::booster::locale; 
}


#endif
