///////////////////////////////////////////////////////////////////////////////
//                                                                             
//  Copyright (C) 2008-2012  Artyom Beilis (Tonkikh) <artyomtnk@yahoo.com>     
//                                                                             
//  See accompanying file COPYING.TXT file for licensing details.
//
///////////////////////////////////////////////////////////////////////////////
#ifndef CPPCMS_LOCALIZATION_H
#define CPPCMS_LOCALIZATION_H

#include <cppcms/config.h>
#include <booster/locale.h>
namespace cppcms {
	namespace locale = ::booster::locale;
}

#endif
