//
//  Copyright (C) 2009-2012 Artyom Beilis (Tonkikh)
//
//  Distributed under the Boost Software License, Version 1.0. (See
//  accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)
//
#ifndef BOOSTER_ASSERT_H
#define BOOSTER_ASSERT_H

#include <assert.h>

#define BOOSTER_ASSERT(x) assert(x)

#endif
