///////////////////////////////////////////////////////////////////////////////
//                                                                             
//  Copyright (C) 2008-2012  Artyom Beilis (Tonkikh) <artyomtnk@yahoo.com>     
//                                                                             
//  See accompanying file COPYING.TXT file for licensing details.
//
///////////////////////////////////////////////////////////////////////////////
#ifndef CPPCMS_CSTDINT_H
#define CPPCMS_CSTDINT_H
#include <booster/cstdint.h>

namespace cppcms {
	using booster::int8_t;
	using booster::uint8_t;
	using booster::uint16_t;
	using booster::int16_t;
	using booster::uint32_t;
	using booster::int32_t;
	using booster::uint64_t;
	using booster::int64_t;
}

#endif // CPPCMS_CSTDINT_H

